import { Injectable } from "@angular/core";
import { LocationStrategy } from "@angular/common";
import { routerLog } from "../trace";
import { Frame } from "ui/frame";
import { isPresent } from "../lang-facade";
const defaultNavOptions = {
    clearHistory: false,
    animated: true
};
export let NSLocationStrategy = class NSLocationStrategy extends LocationStrategy {
    constructor(frame) {
        super();
        this.frame = frame;
        this.states = new Array();
        this.popStateCallbacks = new Array();
        this._isPageNavigationgBack = false;
        routerLog("NSLocationStrategy.constructor()");
    }
    path() {
        let state = this.peekState();
        const result = state ? state.url : "/";
        routerLog("NSLocationStrategy.path(): " + result);
        return result;
    }
    prepareExternalUrl(internal) {
        routerLog("NSLocationStrategy.prepareExternalUrl() internal: " + internal);
        return internal;
    }
    pushState(state, title, url, queryParams) {
        routerLog("NSLocationStrategy.pushState state: " +
            `${state}, title: ${title}, url: ${url}, queryParams: ${queryParams}`);
        this.pushStateInternal(state, title, url, queryParams);
    }
    pushStateInternal(state, title, url, queryParams) {
        let isNewPage = this.states.length === 0;
        this.states.push({
            state: state,
            title: title,
            url: url,
            queryParams: queryParams,
            isPageNavigation: isNewPage
        });
    }
    replaceState(state, title, url, queryParams) {
        if (this.states.length > 0) {
            routerLog("NSLocationStrategy.replaceState changing exisitng state: " +
                `${state}, title: ${title}, url: ${url}, queryParams: ${queryParams}`);
            const topState = this.peekState();
            topState.state = state;
            topState.title = title;
            topState.url = url;
            topState.queryParams = queryParams;
        }
        else {
            routerLog("NSLocationStrategy.replaceState pushing new state: " +
                `${state}, title: ${title}, url: ${url}, queryParams: ${queryParams}`);
            this.pushStateInternal(state, title, url, queryParams);
        }
    }
    forward() {
        throw new Error("NSLocationStrategy.forward() - not implemented");
    }
    back() {
        if (this._isPageNavigationgBack) {
            // We are navigating to the previous page 
            // clear the stack until we get to a page navigation state
            let state = this.states.pop();
            let count = 1;
            while (!(state.isPageNavigation)) {
                state = this.states.pop();
                count++;
            }
            routerLog("NSLocationStrategy.back() while navigating back. States popped: " + count);
            this.callPopState(state, true);
        }
        else {
            let state = this.peekState();
            if (state.isPageNavigation) {
                // This was a page navigation - so navigate through frame.
                routerLog("NSLocationStrategy.back() while not navigating back but top" +
                    " state is page - will call frame.goback()");
                this.frame.goBack();
            }
            else {
                // Nested navigation - just pop the state
                routerLog("NSLocationStrategy.back() while not navigating back but top" +
                    " state is not page - just pop");
                this.callPopState(this.states.pop(), true);
            }
        }
    }
    canGoBack() {
        return this.states.length > 1;
    }
    onPopState(fn) {
        routerLog("NSLocationStrategy.onPopState");
        this.popStateCallbacks.push(fn);
    }
    getBaseHref() {
        routerLog("NSLocationStrategy.getBaseHref()");
        return "";
    }
    callPopState(state, pop = true) {
        const change = { url: state.url, pop: pop };
        for (let fn of this.popStateCallbacks) {
            fn(change);
        }
    }
    peekState() {
        if (this.states.length > 0) {
            return this.states[this.states.length - 1];
        }
        return null;
    }
    toString() {
        return this.states
            .map((v, i) => `${i}.[${v.isPageNavigation ? "PAGE" : "INTERNAL"}] "${v.url}"`)
            .reverse()
            .join("\n");
    }
    // Methods for syncing with page navigation in PageRouterOutlet
    _beginBackPageNavigation() {
        routerLog("NSLocationStrategy.startGoBack()");
        if (this._isPageNavigationgBack) {
            throw new Error("Calling startGoBack while going back.");
        }
        this._isPageNavigationgBack = true;
    }
    _finishBackPageNavigation() {
        routerLog("NSLocationStrategy.finishBackPageNavigation()");
        if (!this._isPageNavigationgBack) {
            throw new Error("Calling endGoBack while not going back.");
        }
        this._isPageNavigationgBack = false;
    }
    _isPageNavigatingBack() {
        return this._isPageNavigationgBack;
    }
    _beginPageNavigation() {
        routerLog("NSLocationStrategy._beginPageNavigation()");
        const lastState = this.peekState();
        if (lastState) {
            lastState.isPageNavigation = true;
        }
        const navOptions = this._currentNavigationOptions || defaultNavOptions;
        if (navOptions.clearHistory) {
            routerLog("NSLocationStrategy._beginPageNavigation clearing states history");
            this.states = [lastState];
        }
        this._currentNavigationOptions = undefined;
        return navOptions;
    }
    _setNavigationOptions(options) {
        this._currentNavigationOptions = {
            clearHistory: isPresent(options.clearHistory) ? options.clearHistory : false,
            animated: isPresent(options.animated) ? options.animated : true,
            transition: options.transition
        };
        routerLog("NSLocationStrategy._setNavigationOptions(" +
            `${JSON.stringify(this._currentNavigationOptions)})`);
    }
    _getSatates() {
        return this.states.slice();
    }
};
NSLocationStrategy = __decorate([
    Injectable(), 
    __metadata('design:paramtypes', [Frame])
], NSLocationStrategy);
//# sourceMappingURL=ns-location-strategy.js.map