import { Directive, HostListener, Input, Optional } from "@angular/core";
import { ActivatedRoute, Router } from "@angular/router";
import { routerLog } from "../trace";
import { PageRoute } from "./page-router-outlet";
import { RouterExtensions } from "./router-extensions";
import { isString } from "utils/types";
/**
 * The nsRouterLink directive lets you link to specific parts of your app.
 *
 * Consider the following route configuration:
 * ```
 * [{ path: "/user", component: UserCmp }]
 * ```
 *
 * When linking to this `User` route, you can write:
 *
 * ```
 * <a [nsRouterLink]="["/user"]">link to user component</a>
 * ```
 *
 * NSRouterLink expects the value to be an array of path segments, followed by the params
 * for that level of routing. For instance `["/team", {teamId: 1}, "user", {userId: 2}]`
 * means that we want to generate a link to `/team;teamId=1/user;userId=2`.
 *
 * The first segment name can be prepended with `/`, `./`, or `../`.
 * If the segment begins with `/`, the router will look up the route from the root of the app.
 * If the segment begins with `./`, or doesn"t begin with a slash, the router will
 * instead look in the current component"s children for the route.
 * And if the segment begins with `../`, the router will go up one level.
 */
export let NSRouterLink = class NSRouterLink {
    constructor(router, navigator, route, pageRoute) {
        this.router = router;
        this.navigator = navigator;
        this.route = route;
        this.pageRoute = pageRoute;
        this.commands = [];
        this.pageTransition = true;
        this.usePageRoute = (this.pageRoute && this.route === this.pageRoute.activatedRoute.getValue());
    }
    get currentRoute() {
        return this.usePageRoute ? this.pageRoute.activatedRoute.getValue() : this.route;
    }
    set params(data) {
        if (Array.isArray(data)) {
            this.commands = data;
        }
        else {
            this.commands = [data];
        }
    }
    onTap() {
        routerLog("nsRouterLink.tapped: " + this.commands + " usePageRoute: " +
            this.usePageRoute + " clearHistory: " + this.clearHistory + " transition: " +
            JSON.stringify(this.pageTransition));
        const transition = this.getTransition();
        let extras = {
            relativeTo: this.currentRoute,
            queryParams: this.queryParams,
            fragment: this.fragment,
            clearHistory: this.clearHistory,
            animated: transition.animated,
            transition: transition.transition
        };
        this.navigator.navigate(this.commands, extras);
    }
    getTransition() {
        if (typeof this.pageTransition === "boolean") {
            return { animated: this.pageTransition };
        }
        else if (isString(this.pageTransition)) {
            if (this.pageTransition === "none" || this.pageTransition === "false") {
                return { animated: false };
            }
            else {
                return { animated: true, transition: { name: this.pageTransition } };
            }
        }
        else {
            return {
                animated: true,
                transition: this.pageTransition
            };
        }
    }
    ngOnChanges(_) {
        this.updateUrlTree();
    }
    updateUrlTree() {
        this.urlTree = this.router.createUrlTree(this.commands, { relativeTo: this.currentRoute, queryParams: this.queryParams, fragment: this.fragment });
    }
};
__decorate([
    Input(), 
    __metadata('design:type', String)
], NSRouterLink.prototype, "target", void 0);
__decorate([
    Input(), 
    __metadata('design:type', Object)
], NSRouterLink.prototype, "queryParams", void 0);
__decorate([
    Input(), 
    __metadata('design:type', String)
], NSRouterLink.prototype, "fragment", void 0);
__decorate([
    Input(), 
    __metadata('design:type', Boolean)
], NSRouterLink.prototype, "clearHistory", void 0);
__decorate([
    Input(), 
    __metadata('design:type', Object)
], NSRouterLink.prototype, "pageTransition", void 0);
__decorate([
    Input("nsRouterLink"), 
    __metadata('design:type', Object), 
    __metadata('design:paramtypes', [Object])
], NSRouterLink.prototype, "params", null);
__decorate([
    HostListener("tap"), 
    __metadata('design:type', Function), 
    __metadata('design:paramtypes', []), 
    __metadata('design:returntype', void 0)
], NSRouterLink.prototype, "onTap", null);
NSRouterLink = __decorate([
    Directive({ selector: "[nsRouterLink]" }),
    __param(3, Optional()), 
    __metadata('design:paramtypes', [Router, RouterExtensions, ActivatedRoute, PageRoute])
], NSRouterLink);
//# sourceMappingURL=ns-router-link.js.map