import { Attribute, Directive, ReflectiveInjector, ViewContainerRef, Inject, ComponentFactoryResolver } from "@angular/core";
import { isPresent } from "../lang-facade";
import { RouterOutletMap, PRIMARY_OUTLET } from "@angular/router";
import { NSLocationStrategy } from "./ns-location-strategy";
import { DEVICE, PAGE_FACTORY } from "../platform-providers";
import { routerLog } from "../trace";
import { DetachedLoader } from "../common/detached-loader";
import { ViewUtil } from "../view-util";
import { Frame } from "ui/frame";
import { Page } from "ui/page";
import { BehaviorSubject } from "rxjs/BehaviorSubject";
export class PageRoute {
    constructor(startRoute) {
        this.activatedRoute = new BehaviorSubject(startRoute);
    }
}
/**
 * Reference Cache
 */
class RefCache {
    constructor() {
        this.cache = new Array();
    }
    push(componentRef, reusedRoute, outletMap, loaderRef) {
        this.cache.push({ componentRef, reusedRoute, outletMap, loaderRef });
    }
    pop() {
        return this.cache.pop();
    }
    peek() {
        return this.cache[this.cache.length - 1];
    }
    get length() {
        return this.cache.length;
    }
}
export let PageRouterOutlet = class PageRouterOutlet {
    constructor(parentOutletMap, containerRef, name, locationStrategy, componentFactoryResolver, resolver, frame, device, pageFactory) {
        this.containerRef = containerRef;
        this.locationStrategy = locationStrategy;
        this.componentFactoryResolver = componentFactoryResolver;
        this.resolver = resolver;
        this.frame = frame;
        this.pageFactory = pageFactory;
        this.refCache = new RefCache();
        this.isInitialPage = true;
        parentOutletMap.registerOutlet(name ? name : PRIMARY_OUTLET, this);
        this.viewUtil = new ViewUtil(device);
        this.detachedLoaderFactory = resolver.resolveComponentFactory(DetachedLoader);
        log("DetachedLoaderFactory loaded");
    }
    get locationInjector() { return this.containerRef.injector; }
    get locationFactoryResolver() { return this.resolver; }
    get isActivated() {
        return !!this.currentActivatedComp;
    }
    get component() {
        if (!this.currentActivatedComp) {
            throw new Error("Outlet is not activated");
        }
        return this.currentActivatedComp.instance;
    }
    get activatedRoute() {
        if (!this.currentActivatedComp) {
            throw new Error("Outlet is not activated");
        }
        return this.currentActivatedRoute;
    }
    deactivate() {
        if (this.locationStrategy._isPageNavigatingBack()) {
            log("PageRouterOutlet.deactivate() while going back - should destroy");
            const poppedItem = this.refCache.pop();
            const poppedRef = poppedItem.componentRef;
            if (this.currentActivatedComp !== poppedRef) {
                throw new Error("Current componentRef is different for cached componentRef");
            }
            this.destroyCacheItem(poppedItem);
            this.currentActivatedComp = null;
        }
        else {
            log("PageRouterOutlet.deactivate() while going forward - do nothing");
        }
    }
    clearRefCache() {
        while (this.refCache.length > 0) {
            this.destroyCacheItem(this.refCache.pop());
        }
    }
    destroyCacheItem(poppedItem) {
        if (isPresent(poppedItem.componentRef)) {
            poppedItem.componentRef.destroy();
        }
        if (isPresent(poppedItem.loaderRef)) {
            poppedItem.loaderRef.destroy();
        }
    }
    /**
     * Called by the Router to instantiate a new component during the commit phase of a navigation.
     * This method in turn is responsible for calling the `routerOnActivate` hook of its child.
     */
    activate(activatedRoute, resolver, injector, providers, outletMap) {
        this.outletMap = outletMap;
        this.currentActivatedRoute = activatedRoute;
        if (this.locationStrategy._isPageNavigatingBack()) {
            this.activateOnGoBack(activatedRoute, outletMap);
        }
        else {
            this.activateOnGoForward(activatedRoute, providers, outletMap, resolver, injector);
        }
    }
    activateOnGoForward(activatedRoute, providers, outletMap, loadedResolver, injector) {
        const factory = this.getComponentFactory(activatedRoute, loadedResolver);
        const pageRoute = new PageRoute(activatedRoute);
        providers = [...providers, ...ReflectiveInjector.resolve([{ provide: PageRoute, useValue: pageRoute }])];
        if (this.isInitialPage) {
            log("PageRouterOutlet.activate() initial page - just load component");
            this.isInitialPage = false;
            const inj = ReflectiveInjector.fromResolvedProviders(providers, injector);
            this.currentActivatedComp = this.containerRef.createComponent(factory, this.containerRef.length, inj, []);
            this.refCache.push(this.currentActivatedComp, pageRoute, outletMap, null);
        }
        else {
            log("PageRouterOutlet.activate() forward navigation - " +
                "create detached loader in the loader container");
            const page = this.pageFactory({
                isNavigation: true,
                componentType: factory.componentType
            });
            const pageResolvedProvider = ReflectiveInjector.resolve([
                { provide: Page, useValue: page }
            ]);
            const childInjector = ReflectiveInjector.fromResolvedProviders([...providers, ...pageResolvedProvider], injector);
            const loaderRef = this.containerRef.createComponent(this.detachedLoaderFactory, this.containerRef.length, childInjector, []);
            this.currentActivatedComp = loaderRef.instance.loadWithFactory(factory);
            this.loadComponentInPage(page, this.currentActivatedComp);
            this.refCache.push(this.currentActivatedComp, pageRoute, outletMap, loaderRef);
        }
    }
    activateOnGoBack(activatedRoute, outletMap) {
        log("PageRouterOutlet.activate() - Back navigation, so load from cache");
        this.locationStrategy._finishBackPageNavigation();
        let cacheItem = this.refCache.peek();
        cacheItem.reusedRoute.activatedRoute.next(activatedRoute);
        this.outletMap = cacheItem.outletMap;
        // HACK: Fill the outlet map provided by the router, with the outlets that we have
        // cached. This is needed because the component is taken from the cache and not
        // created - so it will not register its child router-outlets to the newly created
        // outlet map.
        Object.assign(outletMap, cacheItem.outletMap);
        this.currentActivatedComp = cacheItem.componentRef;
    }
    loadComponentInPage(page, componentRef) {
        // Component loaded. Find its root native view.
        const componentView = componentRef.location.nativeElement;
        // Remove it from original native parent.
        this.viewUtil.removeChild(componentView.parent, componentView);
        // Add it to the new page
        page.content = componentView;
        page.on("navigatedFrom", global.Zone.current.wrap((args) => {
            if (args.isBackNavigation) {
                this.locationStrategy._beginBackPageNavigation();
                this.locationStrategy.back();
            }
        }));
        const navOptions = this.locationStrategy._beginPageNavigation();
        this.frame.navigate({
            create: () => { return page; },
            clearHistory: navOptions.clearHistory,
            animated: navOptions.animated,
            transition: navOptions.transition
        });
        // Clear refCache if navigation with clearHistory
        if (navOptions.clearHistory) {
            this.clearRefCache();
        }
    }
    // NOTE: Using private APIs - potential break point!
    getComponentFactory(activatedRoute, loadedResolver) {
        const snapshot = activatedRoute._futureSnapshot;
        const component = snapshot._routeConfig.component;
        let factory;
        if (loadedResolver) {
            factory = loadedResolver.resolveComponentFactory(component);
        }
        else {
            factory = this.componentFactoryResolver.resolveComponentFactory(component);
        }
        return factory;
    }
};
PageRouterOutlet = __decorate([
    Directive({ selector: "page-router-outlet" }),
    __param(2, Attribute("name")),
    __param(7, Inject(DEVICE)),
    __param(8, Inject(PAGE_FACTORY)), 
    __metadata('design:paramtypes', [RouterOutletMap, ViewContainerRef, String, NSLocationStrategy, ComponentFactoryResolver, ComponentFactoryResolver, Frame, Object, Function])
], PageRouterOutlet);
function log(msg) {
    routerLog(msg);
}
//# sourceMappingURL=page-router-outlet.js.map