import { isString, isDefined } from "utils/types";
import { View } from "ui/core/view";
import { Placeholder } from "ui/placeholder";
import { ContentView } from "ui/content-view";
import { LayoutBase } from "ui/layouts/layout-base";
import { getViewClass, getViewMeta, isKnownView, TEMPLATE } from "./element-registry";
import { getSpecialPropertySetter } from "ui/builder/special-properties";
import { getPropertyByName, withStyleProperty } from "ui/styling/style-property";
import { ValueSource } from "ui/core/dependency-observable";
import { platformNames } from "platform";
import { rendererLog as traceLog, styleError } from "./trace";
const IOS_PREFX = ":ios:";
const ANDROID_PREFX = ":android:";
const whiteSpaceSplitter = /\s+/;
export function isView(view) {
    return view instanceof View;
}
export function isLayout(view) {
    return view instanceof LayoutBase;
}
export function isContentView(view) {
    return view instanceof ContentView;
}
const propertyMaps = new Map();
export class ViewUtil {
    constructor(device) {
        this.isIos = device.os === platformNames.ios;
        this.isAndroid = device.os === platformNames.android;
    }
    insertChild(parent, child, atIndex = -1) {
        if (!parent || child.meta.skipAddToDom) {
            return;
        }
        if (parent.meta && parent.meta.insertChild) {
            parent.meta.insertChild(parent, child, atIndex);
        }
        else if (isLayout(parent)) {
            if (child.parent === parent) {
                let index = parent.getChildIndex(child);
                if (index !== -1) {
                    parent.removeChild(child);
                }
            }
            if (atIndex !== -1) {
                parent.insertChild(child, atIndex);
            }
            else {
                parent.addChild(child);
            }
        }
        else if (isContentView(parent)) {
            // Explicit handling of template anchors inside ContentView
            if (child.meta.isTemplateAnchor) {
                parent._addView(child, atIndex);
            }
            else {
                parent.content = child;
            }
        }
        else if (parent && parent._addChildFromBuilder) {
            parent._addChildFromBuilder(child.nodeName, child);
        }
        else {
        }
    }
    removeChild(parent, child) {
        if (!parent || child.meta.skipAddToDom) {
            return;
        }
        if (parent.meta && parent.meta.removeChild) {
            parent.meta.removeChild(parent, child);
        }
        else if (isLayout(parent)) {
            parent.removeChild(child);
        }
        else if (isContentView(parent)) {
            if (parent.content === child) {
                parent.content = null;
            }
            // Explicit handling of template anchors inside ContentView
            if (child.meta.isTemplateAnchor) {
                parent._removeView(child);
            }
        }
        else if (isView(parent)) {
            parent._removeView(child);
        }
        else {
        }
    }
    getChildIndex(parent, child) {
        if (isLayout(parent)) {
            return parent.getChildIndex(child);
        }
        else if (isContentView(parent)) {
            return child === parent.content ? 0 : -1;
        }
        else {
        }
    }
    createAndAttach(name, viewClass, parent, beforeAttach) {
        const view = new viewClass();
        view.nodeName = name;
        view.meta = getViewMeta(name);
        if (beforeAttach) {
            beforeAttach(view);
        }
        if (parent) {
            this.insertChild(parent, view);
        }
        return view;
    }
    createView(name, parent, beforeAttach) {
        if (isKnownView(name)) {
            const viewClass = getViewClass(name);
            return this.createAndAttach(name, viewClass, parent, beforeAttach);
        }
        else {
            return this.createViewContainer(parent, beforeAttach);
        }
    }
    createText() {
        const text = new Placeholder();
        text.nodeName = "#text";
        text.visibility = "collapse";
        text.meta = getViewMeta("Placeholder");
        return text;
    }
    createViewContainer(parentElement, beforeAttach) {
        traceLog("Creating view container in:" + parentElement);
        const layout = this.createView("ProxyViewContainer", parentElement, beforeAttach);
        layout.nodeName = "ProxyViewContainer";
        return layout;
    }
    createTemplateAnchor(parentElement) {
        const viewClass = getViewClass(TEMPLATE);
        const anchor = this.createAndAttach(TEMPLATE, viewClass, parentElement);
        anchor.templateParent = parentElement;
        anchor.visibility = "collapse";
        traceLog("Created templateAnchor: " + anchor);
        return anchor;
    }
    isXMLAttribute(name) {
        switch (name) {
            case "style": return true;
            case "rows": return true;
            case "columns": return true;
            case "fontAttributes": return true;
            default: return false;
        }
    }
    platformFilter(attribute) {
        let lowered = attribute.toLowerCase();
        if (lowered.indexOf(IOS_PREFX) === 0) {
            if (this.isIos) {
                return attribute.substr(IOS_PREFX.length);
            }
            else {
                return null;
            }
        }
        if (lowered.indexOf(ANDROID_PREFX) === 0) {
            if (this.isAndroid) {
                return attribute.substr(ANDROID_PREFX.length);
            }
            else {
                return null;
            }
        }
        return attribute;
    }
    setProperty(view, attributeName, value) {
        attributeName = this.platformFilter(attributeName);
        if (!attributeName) {
            return;
        }
        if (attributeName.indexOf(".") !== -1) {
            // Handle nested properties
            const properties = attributeName.split(".");
            attributeName = properties[properties.length - 1];
            let propMap = this.getProperties(view);
            let i = 0;
            while (i < properties.length - 1 && isDefined(view)) {
                let prop = properties[i];
                if (propMap.has(prop)) {
                    prop = propMap.get(prop);
                }
                view = view[prop];
                propMap = this.getProperties(view);
                i++;
            }
        }
        if (isDefined(view)) {
            this.setPropertyInternal(view, attributeName, value);
        }
    }
    setPropertyInternal(view, attributeName, value) {
        traceLog("Setting attribute: " + attributeName);
        let specialSetter = getSpecialPropertySetter(attributeName);
        let propMap = this.getProperties(view);
        if (attributeName === "class") {
            this.setClasses(view, value);
        }
        else if (this.isXMLAttribute(attributeName)) {
            view._applyXmlAttribute(attributeName, value);
        }
        else if (specialSetter) {
            specialSetter(view, value);
        }
        else if (propMap.has(attributeName)) {
            // We have a lower-upper case mapped property.
            let propertyName = propMap.get(attributeName);
            view[propertyName] = this.convertValue(value);
        }
        else {
            // Unknown attribute value -- just set it to our object as is.
            view[attributeName] = this.convertValue(value);
        }
    }
    convertValue(value) {
        if (typeof (value) !== "string" || value === "") {
            return value;
        }
        let valueAsNumber = +value;
        if (!isNaN(valueAsNumber)) {
            return valueAsNumber;
        }
        else if (value && (value.toLowerCase() === "true" || value.toLowerCase() === "false")) {
            return value.toLowerCase() === "true" ? true : false;
        }
        else {
            return value;
        }
    }
    getProperties(instance) {
        let type = instance && instance.constructor;
        if (!type) {
            return new Map();
        }
        if (!propertyMaps.has(type)) {
            let propMap = new Map();
            for (let propName in instance) {
                propMap.set(propName.toLowerCase(), propName);
            }
            propertyMaps.set(type, propMap);
        }
        return propertyMaps.get(type);
    }
    cssClasses(view) {
        if (!view.ngCssClasses) {
            view.ngCssClasses = new Map();
        }
        return view.ngCssClasses;
    }
    addClass(view, className) {
        this.cssClasses(view).set(className, true);
        this.syncClasses(view);
    }
    removeClass(view, className) {
        this.cssClasses(view).delete(className);
        this.syncClasses(view);
    }
    setClasses(view, classesValue) {
        let classes = classesValue.split(whiteSpaceSplitter);
        this.cssClasses(view).clear();
        classes.forEach((className) => this.cssClasses(view).set(className, true));
        this.syncClasses(view);
    }
    syncClasses(view) {
        let classValue = Array.from(this.cssClasses(view).keys()).join(" ");
        view.cssClass = classValue;
    }
    resolveCssValue(styleValue) {
        return styleValue;
    }
    setStyleValue(view, property, value) {
        try {
            if (value === null) {
                view.style._resetValue(property, ValueSource.Local);
            }
            else {
                view.style._setValue(property, value, ValueSource.Local);
            }
        }
        catch (ex) {
            styleError("Error setting property: " + property.name + " view: " + view +
                " value: " + value + " " + ex);
        }
    }
    setStyleProperty(view, styleName, styleValue) {
        traceLog("setStyleProperty: " + styleName + " = " + styleValue);
        let name = styleName;
        let resolvedValue = this.resolveCssValue(styleValue);
        withStyleProperty(name, resolvedValue, (property, value) => {
            if (isString(property)) {
                // Fall back to resolving property by name.
                const resolvedProperty = getPropertyByName(name);
                if (resolvedProperty) {
                    this.setStyleValue(view, resolvedProperty, resolvedValue);
                }
                else {
                    traceLog("Unknown style property: " + styleName);
                }
            }
            else {
                const resolvedProperty = property;
                this.setStyleValue(view, resolvedProperty, value);
            }
        });
    }
}
//# sourceMappingURL=view-util.js.map